"use strict";
var Test = require("./example-test");
var match_error_1 = require("./errors/match-error");
require("reflect-metadata");
var currentTestId = 0;
var getTestDescription = function (test, testCaseArguments) {
    var testDescription = currentTestId + " - " + test.description;
    if (testCaseArguments !== undefined) {
        testDescription += " [ " + testCaseArguments.map(function (x) { return JSON.stringify(x) || "undefined"; }).join(", ") + " ]";
    }
    return testDescription;
};
var handleError = function (error, test, testCaseArguments) {
    process.stdout.write("not ok " + getTestDescription(test, testCaseArguments) + "\n");
    if (error instanceof match_error_1.MatchError) {
        process.stdout.write(" ---\n   message: \"" + error.message + "\"\n   severity: fail\n   data:\n     got: " + JSON.stringify(error.actualValue) + "\n     expect: " + JSON.stringify(error.expectedValue) + "\n ...\n");
    }
    else {
        console.log(error);
        process.stdout.write("# Unknown Error\n");
    }
    teardown();
};
var notifySuccess = function (test, testCaseArguments) {
    process.stdout.write("ok " + getTestDescription(test, testCaseArguments) + "\n");
    teardown();
};
var teardown = function () {
    var testFixture = testFixtures[currentTestFixtureIndex];
    var teardownFunctions = Reflect.getMetadata("alsatian:teardown", testFixture.fixture);
    if (teardownFunctions) {
        teardownFunctions.forEach(function (teardownFunction) {
            testFixture.fixture[teardownFunction].call(testFixture.fixture);
        });
    }
    runNextTestCase();
};
var testFixtureKeys = Object.keys(Test);
var testFixtures = [];
var testsFocussed = false;
testFixtureKeys.forEach(function (testFixtureKey) {
    if (Reflect.getMetadata("alsatian:ignore", Test[testFixtureKey])) {
        return;
    }
    var testFixture = {};
    testFixture.fixture = new Test[testFixtureKey]();
    var tests = Reflect.getMetadata("alsatian:tests", testFixture.fixture);
    if (!tests || tests.length === 0) {
        return;
    }
    var focusFixture = Reflect.getMetadata("alsatian:focus", Test[testFixtureKey]);
    testFixture.tests = [];
    tests.forEach(function (test) {
        if (Reflect.getMetadata("alsatian:ignore", testFixture.fixture, test.key)) {
            return;
        }
        var focusTest = Reflect.getMetadata("alsatian:focus", testFixture.fixture, test.key);
        test.focussed = focusFixture || focusTest;
        testsFocussed = testsFocussed || test.focussed;
        testFixture.tests.push(test);
        if (!test.description) {
            test.description = test.key;
        }
        var testCases = Reflect.getMetadata("alsatian:testcases", testFixture.fixture, test.key);
        test.testCases = [];
        if (!testCases) {
            test.testCases.push([]);
        }
        else {
            testCases.forEach(function (testCase) {
                test.testCases.push(testCase);
            });
        }
    });
    testFixtures.push(testFixture);
});
if (testFixtures.length === 0) {
    process.stderr.write("no tests to run\n");
    process.exit(1);
}
if (testsFocussed) {
    testFixtures = testFixtures.map(function (x) {
        x.tests = x.tests.filter(function (y) { return y.focussed; });
        return x;
    });
}
var totalTestCount = testFixtures.map(function (x) { return x.tests.map(function (y) { return y.testCases.length; }); }).reduce(function (a, b) { return a + b; }).reduce(function (c, d) { return c + d; });
process.stdout.write("TAP version 13\n");
process.stdout.write("1.." + totalTestCount + "\n");
var currentTestFixtureIndex = 0;
var currentTestIndex = 0;
var currentTestCaseIndex = 0;
var exit = function () {
    process.exit(0);
};
var runNextTestFixture = function () {
    currentTestFixtureIndex++;
    currentTestIndex = 0;
    currentTestCaseIndex = 0;
    if (!testFixtures[currentTestFixtureIndex]) {
        exit();
    }
    else {
        runTest(testFixtures[currentTestFixtureIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex].testCases[currentTestCaseIndex].arguments);
    }
};
var runNextTest = function () {
    currentTestIndex++;
    currentTestCaseIndex = 0;
    if (!testFixtures[currentTestFixtureIndex].tests[currentTestIndex]) {
        runNextTestFixture();
    }
    else {
        runTest(testFixtures[currentTestFixtureIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex].testCases[currentTestCaseIndex].arguments);
    }
};
var runNextTestCase = function () {
    currentTestCaseIndex++;
    if (!testFixtures[currentTestFixtureIndex].tests[currentTestIndex].testCases[currentTestCaseIndex]) {
        runNextTest();
    }
    else {
        runTest(testFixtures[currentTestFixtureIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex].testCases[currentTestCaseIndex].arguments);
    }
};
var runTest = function (testFixture, test, testCaseArguments) {
    currentTestId++;
    var setupFunctions = Reflect.getMetadata("alsatian:setup", testFixture.fixture);
    if (setupFunctions) {
        setupFunctions.forEach(function (setupFunction) {
            testFixture.fixture[setupFunction].call(testFixture.fixture);
        });
    }
    try {
        if (test.isAsync) {
            var timeout_1 = false;
            var promise = testFixture.fixture[test.key].apply(testFixture.fixture, testCaseArguments);
            promise.then(function () {
                if (!timeout_1) {
                    notifySuccess(test, testCaseArguments);
                }
            })
                .catch(function (error) {
                handleError(error, test, testCaseArguments);
            });
            var timeoutCheck = setTimeout(function () {
                timeout_1 = true;
                handleError(new match_error_1.MatchError("longer than 500ms", "less than 500ms", "The test exceeded the given timeout."), test, testCaseArguments);
            }, 500);
        }
        else {
            testFixture.fixture[test.key].apply(testFixture, testCaseArguments);
            notifySuccess(test, testCaseArguments);
        }
    }
    catch (error) {
        handleError(error, test, testCaseArguments);
    }
};
runTest(testFixtures[currentTestFixtureIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex], testFixtures[currentTestFixtureIndex].tests[currentTestIndex].testCases[currentTestCaseIndex].arguments);
//# sourceMappingURL=runner.js.map